@@ -49,10 +49,6 @@
 #include "sudo.h"
 #include "sudo_auth.h"
 
-#ifndef lint
-__unused static const char rcsid[] = "$Sudo: fwtk.c,v 1.29 2008/11/09 14:13:13 millert Exp $";
-#endif /* lint */
-
 int
 fwtk_init(pw, promptp, auth)
     struct passwd *pw;
