@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2005 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 1999-2005, 2007 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -47,10 +47,6 @@
 #include "sudo.h"
 #include "sudo_auth.h"
 
-#ifndef lint
-__unused static const char rcsid[] = "$Sudo: kerb4.c,v 1.11.2.2 2007/06/12 01:28:42 millert Exp $";
-#endif /* lint */
-
 int
 kerb4_init(pw, promptp, auth)
     struct passwd *pw;
@@ -68,7 +64,7 @@ kerb4_init(pw, promptp, auth)
 	return(AUTH_FAILURE);
 
     /* Stash a pointer to the realm (used in kerb4_verify) */
-    auth->data = (VOID *) realm;
+    auth->data = (void *) realm;
 
     return(AUTH_SUCCESS);
 }
