@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2005 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 1999-2005, 2007 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -44,11 +44,6 @@
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif /* HAVE_UNISTD_H */
-#ifdef HAVE_ERR_H
-# include <err.h>
-#else
-# include "emul/err.h"
-#endif /* HAVE_ERR_H */
 #include <pwd.h>
 
 #include <sdi_athd.h>
@@ -58,6 +53,10 @@
 #include "sudo.h"
 #include "sudo_auth.h"
 
+#ifndef lint
+__unused static const char rcsid[] = "$Sudo: securid.c,v 1.18 2008/11/09 14:13:13 millert Exp $";
+#endif /* lint */
+
 union config_record configure;
 
 int
@@ -68,7 +67,7 @@ securid_init(pw, promptp, auth)
 {
     static struct SD_CLIENT sd_dat;		/* SecurID data block */
 
-    auth->data = (VOID *) &sd_dat;		/* For method-specific data */
+    auth->data = (void *) &sd_dat;		/* For method-specific data */
 
     if (creadcfg() == 0)
 	return(AUTH_SUCCESS);
@@ -90,7 +89,7 @@ securid_setup(pw, promptp, auth)
 	strlcpy(sd->username, pw->pw_name, 32);
 	return(AUTH_SUCCESS);
     } else {
-	warnx("unable to contact the SecurID server");
+	warningx("unable to contact the SecurID server");
 	return(AUTH_FATAL);
     }
 }
