@@ -78,10 +78,6 @@
 
 #include <compat.h>
 
-#ifndef lint
-__unused static const char rcsid[] = "$Sudo: snprintf.c,v 1.22 2008/11/09 14:13:12 millert Exp $";
-#endif /* lint */
-
 static int xxxprintf	 __P((char **, size_t, int, const char *, va_list));
 
 /*
