@@ -22,6 +22,11 @@
 #include <config.h>
 #include <compat.h>
 
+#ifndef lint
+__unused static const char rcsid[] = "$Sudo: strlcat.c,v 1.7 2005/02/12 22:56:06 millert Exp $";
+#endif /* lint */
+
+
 /*
  * Appends src to string dst of size siz (unlike strncat, siz is the
  * full size of dst, not space left).  At most siz-1 characters
