@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2005 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2003-2005, 2007 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -19,17 +19,13 @@
 #include <config.h>
 #include <compat.h>
 
-#ifndef lint
-__unused static const char rcsid[] = "$Sudo: zero_bytes.c,v 1.2.2.2 2007/06/12 01:28:42 millert Exp $";
-#endif /* lint */
-
 /*
  * Like bzero(3) but with a volatile pointer.  The hope is that
  * the compiler will not be able to optimize away this function.
  */
 void
 zero_bytes(v, n)
-    volatile VOID *v;
+    volatile void *v;
     size_t n;
 {
     volatile char *p, *ep;
